/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package iocard;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 *
 * @author www.leniwiec.org 
 */
public class SerialHandler {
    private InputStream in;
    private OutputStream out;
    private CommPort commPort;

    public SerialHandler()
    {
        super();
    }

    void disconnect()
    {
        this.commPort.close();
    }//end of disconnect()

    void connect(String portName) throws Exception
    {
        CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        if ( portIdentifier.isCurrentlyOwned() )
        {
            System.out.println("Error: Port is currently in use");
        }
        else
        {
            this.commPort = portIdentifier.open(this.getClass().getName(),2000);

            if ( commPort instanceof SerialPort )
            {
                SerialPort serialPort = (SerialPort) commPort;
                serialPort.setSerialPortParams(9600,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);

                this.in = serialPort.getInputStream();
                this.out = serialPort.getOutputStream();
            }
            else
            {
                System.out.println("Error: Only serial ports are handled by this example.");
            }
        }
    } // end of connect()

    static void listPorts()
    {
        java.util.Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();
        while ( portEnum.hasMoreElements() )
        {
            CommPortIdentifier portIdentifier = portEnum.nextElement();
            System.out.println(portIdentifier.getName()  +  " - " +  getPortTypeName(portIdentifier.getPortType()) );
        }
    } //end of listPorts()
    
    static String getPortTypeName ( int portType )
    {
        switch ( portType )
        {
            case CommPortIdentifier.PORT_I2C:
                return "I2C";
            case CommPortIdentifier.PORT_PARALLEL:
                return "Parallel";
            case CommPortIdentifier.PORT_RAW:
                return "Raw";
            case CommPortIdentifier.PORT_RS485:
                return "RS485";
            case CommPortIdentifier.PORT_SERIAL:
                return "Serial";
            default:
                return "unknown type";
        }
    }//end of getPortTypeName()

    public InputStream getIn()
    {
        return this.in;
    }//end of getIn()

    public OutputStream getOut()
    {
        return this.out;
    }//end of getOut()

    public void setCmd(byte bIn[])
    {
        byte[] b = new byte[10];
        
        b[0]='s';
        b[1]='0';
        b[2]='0';
        b[3]='0';
        b[4]='0';
        b[5]='0';
        b[6]='0';
        b[7]='0';
        b[8]='0';
        b[9]='s';

        for(int i=0; i < 8 ; i++)
            b[i+1] = bIn[i];

        try {
            for(int i=0; i < 10 ; i++)
            {
                this.out.write(b[i]);
            }
        } catch (IOException ex) {
           ex.printStackTrace();
        }
    }//end of setCmd()

    public byte[] readStatus()
    {
        byte[] b = new byte[8];
        
        try {
            int toSkip = this.in.available();
            if(toSkip > 0)
                this.in.skip(toSkip);
        
            this.out.write('g');

            for(int i=0; i < 8 ; i++)
                 b[i] = (byte) this.in.read();

        } catch (IOException ex) {
            ex.printStackTrace();
        }

        return b;
    }//end of readStatus()
}
