<?php

/**
 * IoCart
 *
 * @author	www.leniwiec.org - http://www.leniwiec.org <root@leniwiec.org> 
 */
class IoCart
{
	/**
	 * The connection timeout, in seconds. 
	 */
	const SOCKET_TIMEOUT = 60;

	/**
	 * Deley time (in microseconds) between
	 * operation on port.
	 */
	const PORT_DELAY = 80000;

	/**
	 * COM port 
	 *
	 * @var	resource
	 */
	private $resRs232 = null;

	/**
	 * Holds the system level error number that 
	 * occurred in the system-level connect() call. 
	 *
	 * @var	integer
	 */
	private $intErrno = null;

	/**
	 * The error message as a string. 
	 *
	 * @var	string
	 */
	private $strError = null;


	/**
	 * Constructor
	 *
	 * Create connection to socket by given params
	 *
	 * @param	$strHost
	 * @param	$strPort
	 * @throw	IoCart_Exception
	 */
	public function __construct($strHost, $strPort)
	{
		$this->resRs232 = fsockopen($strHost, $strPort, &$this->intErrno, &$this->strError, self::SOCKET_TIMEOUT);

		if($this->intErrno == 0 && $this->resRs232 === false)
		{
			throw new IoCart_Exception('Problem initializing the socket');
		}

		if($this->intErrno != 0)
		{
			throw new IoCart_Exception('fsockopen: '.$this->strError);
		}
	} // end of __construct()

	/**
	 * Destructor
	 *
	 * @param	void
	 */
	public function __destruct()
	{	
		$this->close();
	} // end of __destruct()

	/**
	 * Close connection
	 *
	 * @param	void
	 * @return	void
	 */
	public function close()
	{
		fclose($this->resRs232);
	} // end of close()

	/**
	 * Get status from device
	 *
	 * @param	void
	 * @return	string				8character string represents 8bit word
	 * @throw	IoCart_Exception
	 */
	public function getStatus()
	{
		if($this->resRs232 === null || $this->resRs232 === false)
			throw new IoCart_Exception('Not connected to device');

		/* send get comand to deveci and wait for response */
		fwrite($this->resRs232, 'g', 1);
		//usleep(self::PORT_DELAY);

		/* read all response from device */
		$strBuff = fgets($this->resRs232, 1024);

		/* clear output */
		$strBuff = substr($strBuff, -9, 8);

		/* check response */
		for($intI=0; $intI < 8; $intI++)
		{
			if($strBuff[$intI]!='1' && $strBuff[$intI]!='0')
				throw new IoCart_Exception(__METHOD__.': Device return strange response');
		}

		return $strBuff;
	} // end of getStatus()

	/**
	 * Check given port status 
	 *
	 * @param	integer	$intPortNumber		Device port number
	 * @return	boolean
	 * @throw	IoCart_Exception
	 */
	public function checkStatus($intPortNumber)
	{
		if($intPortNumber<1 || $intPortNumber>8)
			throw new IoCart_Exception(__METHOD__.': port number should be integer value between 1 and 8');

		$strStatus = $this->getStatus();

		if($strStatus[$intPortNumber-1] == '1')
			return true;
		else
			return false;
	} // end of checkStatus()

	/**
	 * Set port in state
	 *
	 * @param	integer	$intPort	Port number (value from 1 to 8)
	 * @param	boolean	$bolState	State of port to set
	 * @return	void
	 * @throw	IoCart_Exception
	 */
	public function setPort($intPort, $bolState)
	{
		if($intPort<1 || $intPort>8)
			throw new IoCart_Exception(__METHOD__.': port number should be integer value between 1 and 8');

		if(!is_bool($bolState))
			throw new IoCart_Exception(__METHOD__.': state should be boolean value');

		/* get current device status */
		$strStatus = $this->getStatus();

		/* prepare and send cmd */
		$strStatus[$intPort-1] = $bolState ? '1' : '0';
		fwrite($this->resRs232, 's'.$strStatus.'s',10);

		/* read all response from device */
		$strBuff = fgets($this->resRs232, 1024);

		/* clear output */
		$strBuff = substr($strBuff, -3, 2);

		/* check response */
		if($strBuff != 'OK')
				throw new IoCart_Exception(__METHOD__.': Device return strange response');
	} // end of setPort()

	/**
	 * Set whole byte 
	 *
	 * @param	string	$strByte	String representation of byte
	 * @return	void
	 * @throw	IoCart_Exception
	 */
	public function set($strByte)
	{
		if(strlen($strByte)!=8)
			throw new IoCart_Exception(__METHOD__.': string representation of byte should have exacly 8 characters');

		/* check response */
		for($intI=0; $intI < 8; $intI++)
		{
			if($strByte[$intI]!='1' && $strByte[$intI]!='0')
				throw new IoCart_Exception(__METHOD__.': Wrong input byte format');
		}

		/* prepare and send cmd */
		fwrite($this->resRs232, 's'.$strByte.'s',10);

		/* read all response from device */
		$strBuff = fgets($this->resRs232, 1024);

		/* clear output */
		$strBuff = substr($strBuff, -3, 2);

		/* check response */
		if($strBuff != 'OK')
				throw new IoCart_Exception(__METHOD__.': Device return strange response');
	} // end of set()

} // end of IoCart class

class IoCart_Exception extends Exception
{

} // end of IoCart_Exception class


